Package: clock
Title: Date-Time Types and Tools
Version: 0.7.4
Authors@R: c(
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Provides a comprehensive library for date-time manipulations
    using a new family of orthogonal date-time classes (durations, time
    points, zoned-times, and calendars) that partition responsibilities so
    that the complexities of time zones are only considered when they are
    really needed. Capabilities include: date-time parsing, formatting,
    arithmetic, extraction and updating of components, and rounding.
License: MIT + file LICENSE
URL: https://clock.r-lib.org, https://github.com/r-lib/clock
BugReports: https://github.com/r-lib/clock/issues
Depends: R (>= 4.0.0)
Imports: cli (>= 3.6.4), lifecycle (>= 1.0.4), rlang (>= 1.1.5), tzdb
        (>= 0.5.0), vctrs (>= 0.6.5)
Suggests: covr, knitr, magrittr, pillar, rmarkdown, slider (>= 0.3.2),
        testthat (>= 3.0.0), withr
LinkingTo: cpp11 (>= 0.5.2), tzdb (>= 0.5.0)
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/Needs/website: lubridate, tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-01-13 14:52:13 UTC; davis
Author: Davis Vaughan [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2026-01-13 16:10:02 UTC
