# Generated by roxygen2: do not edit by hand

S3method("[",clock_rcrd)
S3method("[[",clock_rcrd)
S3method("names<-",clock_rcrd)
S3method(add_days,Date)
S3method(add_days,POSIXt)
S3method(add_days,clock_calendar)
S3method(add_days,clock_duration)
S3method(add_days,clock_time_point)
S3method(add_days,clock_weekday)
S3method(add_days,clock_zoned_time)
S3method(add_days,default)
S3method(add_hours,POSIXt)
S3method(add_hours,clock_calendar)
S3method(add_hours,clock_duration)
S3method(add_hours,clock_time_point)
S3method(add_hours,clock_zoned_time)
S3method(add_hours,default)
S3method(add_microseconds,clock_calendar)
S3method(add_microseconds,clock_duration)
S3method(add_microseconds,clock_time_point)
S3method(add_microseconds,clock_zoned_time)
S3method(add_microseconds,default)
S3method(add_milliseconds,clock_calendar)
S3method(add_milliseconds,clock_duration)
S3method(add_milliseconds,clock_time_point)
S3method(add_milliseconds,clock_zoned_time)
S3method(add_milliseconds,default)
S3method(add_minutes,POSIXt)
S3method(add_minutes,clock_calendar)
S3method(add_minutes,clock_duration)
S3method(add_minutes,clock_time_point)
S3method(add_minutes,clock_zoned_time)
S3method(add_minutes,default)
S3method(add_months,Date)
S3method(add_months,POSIXt)
S3method(add_months,clock_duration)
S3method(add_months,clock_time_point)
S3method(add_months,clock_year_month_day)
S3method(add_months,clock_year_month_weekday)
S3method(add_months,clock_zoned_time)
S3method(add_months,default)
S3method(add_nanoseconds,clock_calendar)
S3method(add_nanoseconds,clock_duration)
S3method(add_nanoseconds,clock_time_point)
S3method(add_nanoseconds,clock_zoned_time)
S3method(add_nanoseconds,default)
S3method(add_quarters,Date)
S3method(add_quarters,POSIXt)
S3method(add_quarters,clock_duration)
S3method(add_quarters,clock_time_point)
S3method(add_quarters,clock_year_month_day)
S3method(add_quarters,clock_year_month_weekday)
S3method(add_quarters,clock_year_quarter_day)
S3method(add_quarters,clock_zoned_time)
S3method(add_quarters,default)
S3method(add_seconds,POSIXt)
S3method(add_seconds,clock_calendar)
S3method(add_seconds,clock_duration)
S3method(add_seconds,clock_time_point)
S3method(add_seconds,clock_zoned_time)
S3method(add_seconds,default)
S3method(add_weeks,Date)
S3method(add_weeks,POSIXt)
S3method(add_weeks,clock_calendar)
S3method(add_weeks,clock_duration)
S3method(add_weeks,clock_time_point)
S3method(add_weeks,clock_zoned_time)
S3method(add_weeks,default)
S3method(add_years,Date)
S3method(add_years,POSIXt)
S3method(add_years,clock_duration)
S3method(add_years,clock_iso_year_week_day)
S3method(add_years,clock_time_point)
S3method(add_years,clock_year_day)
S3method(add_years,clock_year_month_day)
S3method(add_years,clock_year_month_weekday)
S3method(add_years,clock_year_quarter_day)
S3method(add_years,clock_year_week_day)
S3method(add_years,clock_zoned_time)
S3method(add_years,default)
S3method(as.Date,clock_calendar)
S3method(as.Date,clock_time_point)
S3method(as.Date,clock_zoned_time)
S3method(as.POSIXct,clock_calendar)
S3method(as.POSIXct,clock_naive_time)
S3method(as.POSIXct,clock_sys_time)
S3method(as.POSIXct,clock_zoned_time)
S3method(as.POSIXlt,clock_calendar)
S3method(as.POSIXlt,clock_naive_time)
S3method(as.POSIXlt,clock_sys_time)
S3method(as.POSIXlt,clock_zoned_time)
S3method(as.character,clock_duration)
S3method(as.character,clock_iso_year_week_day)
S3method(as.character,clock_naive_time)
S3method(as.character,clock_sys_time)
S3method(as.character,clock_weekday)
S3method(as.character,clock_year_day)
S3method(as.character,clock_year_month_day)
S3method(as.character,clock_year_month_weekday)
S3method(as.character,clock_year_quarter_day)
S3method(as.character,clock_year_week_day)
S3method(as.character,clock_zoned_time)
S3method(as.double,clock_duration)
S3method(as.integer,clock_duration)
S3method(as_date,Date)
S3method(as_date,POSIXt)
S3method(as_date,clock_calendar)
S3method(as_date,clock_time_point)
S3method(as_date,clock_zoned_time)
S3method(as_date_time,Date)
S3method(as_date_time,POSIXt)
S3method(as_date_time,clock_calendar)
S3method(as_date_time,clock_naive_time)
S3method(as_date_time,clock_sys_time)
S3method(as_date_time,clock_zoned_time)
S3method(as_duration,clock_duration)
S3method(as_duration,clock_time_point)
S3method(as_iso_year_week_day,Date)
S3method(as_iso_year_week_day,POSIXt)
S3method(as_iso_year_week_day,clock_calendar)
S3method(as_iso_year_week_day,clock_iso_year_week_day)
S3method(as_iso_year_week_day,clock_time_point)
S3method(as_iso_year_week_day,default)
S3method(as_naive_time,Date)
S3method(as_naive_time,POSIXt)
S3method(as_naive_time,clock_duration)
S3method(as_naive_time,clock_iso_year_week_day)
S3method(as_naive_time,clock_naive_time)
S3method(as_naive_time,clock_sys_time)
S3method(as_naive_time,clock_year_day)
S3method(as_naive_time,clock_year_month_day)
S3method(as_naive_time,clock_year_month_weekday)
S3method(as_naive_time,clock_year_quarter_day)
S3method(as_naive_time,clock_year_week_day)
S3method(as_naive_time,clock_zoned_time)
S3method(as_naive_time,default)
S3method(as_sys_time,Date)
S3method(as_sys_time,POSIXt)
S3method(as_sys_time,clock_duration)
S3method(as_sys_time,clock_iso_year_week_day)
S3method(as_sys_time,clock_naive_time)
S3method(as_sys_time,clock_sys_time)
S3method(as_sys_time,clock_year_day)
S3method(as_sys_time,clock_year_month_day)
S3method(as_sys_time,clock_year_month_weekday)
S3method(as_sys_time,clock_year_quarter_day)
S3method(as_sys_time,clock_year_week_day)
S3method(as_sys_time,clock_zoned_time)
S3method(as_sys_time,default)
S3method(as_weekday,Date)
S3method(as_weekday,POSIXt)
S3method(as_weekday,clock_calendar)
S3method(as_weekday,clock_time_point)
S3method(as_weekday,clock_weekday)
S3method(as_year_day,Date)
S3method(as_year_day,POSIXt)
S3method(as_year_day,clock_calendar)
S3method(as_year_day,clock_time_point)
S3method(as_year_day,clock_year_day)
S3method(as_year_day,default)
S3method(as_year_month_day,Date)
S3method(as_year_month_day,POSIXt)
S3method(as_year_month_day,clock_calendar)
S3method(as_year_month_day,clock_time_point)
S3method(as_year_month_day,clock_year_month_day)
S3method(as_year_month_day,default)
S3method(as_year_month_weekday,Date)
S3method(as_year_month_weekday,POSIXt)
S3method(as_year_month_weekday,clock_calendar)
S3method(as_year_month_weekday,clock_time_point)
S3method(as_year_month_weekday,clock_year_month_weekday)
S3method(as_year_month_weekday,default)
S3method(as_year_quarter_day,Date)
S3method(as_year_quarter_day,POSIXt)
S3method(as_year_quarter_day,clock_calendar)
S3method(as_year_quarter_day,clock_time_point)
S3method(as_year_quarter_day,clock_year_quarter_day)
S3method(as_year_quarter_day,default)
S3method(as_year_week_day,Date)
S3method(as_year_week_day,POSIXt)
S3method(as_year_week_day,clock_calendar)
S3method(as_year_week_day,clock_time_point)
S3method(as_year_week_day,clock_year_week_day)
S3method(as_year_week_day,default)
S3method(as_zoned_time,Date)
S3method(as_zoned_time,POSIXt)
S3method(as_zoned_time,clock_naive_time)
S3method(as_zoned_time,clock_sys_time)
S3method(as_zoned_time,clock_zoned_time)
S3method(as_zoned_time,default)
S3method(calendar_count_between,clock_calendar)
S3method(calendar_count_between,clock_iso_year_week_day)
S3method(calendar_count_between,clock_year_day)
S3method(calendar_count_between,clock_year_month_day)
S3method(calendar_count_between,clock_year_month_weekday)
S3method(calendar_count_between,clock_year_quarter_day)
S3method(calendar_count_between,clock_year_week_day)
S3method(calendar_end,clock_calendar)
S3method(calendar_end,clock_iso_year_week_day)
S3method(calendar_end,clock_year_day)
S3method(calendar_end,clock_year_month_day)
S3method(calendar_end,clock_year_month_weekday)
S3method(calendar_end,clock_year_quarter_day)
S3method(calendar_end,clock_year_week_day)
S3method(calendar_group,clock_calendar)
S3method(calendar_group,clock_iso_year_week_day)
S3method(calendar_group,clock_year_day)
S3method(calendar_group,clock_year_month_day)
S3method(calendar_group,clock_year_month_weekday)
S3method(calendar_group,clock_year_quarter_day)
S3method(calendar_group,clock_year_week_day)
S3method(calendar_is_precision,clock_iso_year_week_day)
S3method(calendar_is_precision,clock_year_day)
S3method(calendar_is_precision,clock_year_month_day)
S3method(calendar_is_precision,clock_year_month_weekday)
S3method(calendar_is_precision,clock_year_quarter_day)
S3method(calendar_is_precision,clock_year_week_day)
S3method(calendar_leap_year,clock_calendar)
S3method(calendar_leap_year,clock_iso_year_week_day)
S3method(calendar_leap_year,clock_year_day)
S3method(calendar_leap_year,clock_year_month_day)
S3method(calendar_leap_year,clock_year_month_weekday)
S3method(calendar_leap_year,clock_year_quarter_day)
S3method(calendar_leap_year,clock_year_week_day)
S3method(calendar_month_factor,clock_calendar)
S3method(calendar_month_factor,clock_year_month_day)
S3method(calendar_month_factor,clock_year_month_weekday)
S3method(calendar_name,clock_iso_year_week_day)
S3method(calendar_name,clock_year_day)
S3method(calendar_name,clock_year_month_day)
S3method(calendar_name,clock_year_month_weekday)
S3method(calendar_name,clock_year_quarter_day)
S3method(calendar_name,clock_year_week_day)
S3method(calendar_narrow,clock_calendar)
S3method(calendar_narrow,clock_iso_year_week_day)
S3method(calendar_narrow,clock_year_day)
S3method(calendar_narrow,clock_year_month_day)
S3method(calendar_narrow,clock_year_month_weekday)
S3method(calendar_narrow,clock_year_quarter_day)
S3method(calendar_narrow,clock_year_week_day)
S3method(calendar_precision,clock_calendar)
S3method(calendar_start,clock_calendar)
S3method(calendar_start,clock_iso_year_week_day)
S3method(calendar_start,clock_year_day)
S3method(calendar_start,clock_year_month_day)
S3method(calendar_start,clock_year_month_weekday)
S3method(calendar_start,clock_year_quarter_day)
S3method(calendar_start,clock_year_week_day)
S3method(calendar_widen,clock_calendar)
S3method(calendar_widen,clock_iso_year_week_day)
S3method(calendar_widen,clock_year_day)
S3method(calendar_widen,clock_year_month_day)
S3method(calendar_widen,clock_year_month_weekday)
S3method(calendar_widen,clock_year_quarter_day)
S3method(calendar_widen,clock_year_week_day)
S3method(clock_maximum,clock_duration)
S3method(clock_maximum,clock_iso_year_week_day)
S3method(clock_maximum,clock_time_point)
S3method(clock_maximum,clock_year_day)
S3method(clock_maximum,clock_year_month_day)
S3method(clock_maximum,clock_year_month_weekday)
S3method(clock_maximum,clock_year_quarter_day)
S3method(clock_maximum,clock_year_week_day)
S3method(clock_minimum,clock_duration)
S3method(clock_minimum,clock_iso_year_week_day)
S3method(clock_minimum,clock_time_point)
S3method(clock_minimum,clock_year_day)
S3method(clock_minimum,clock_year_month_day)
S3method(clock_minimum,clock_year_month_weekday)
S3method(clock_minimum,clock_year_quarter_day)
S3method(clock_minimum,clock_year_week_day)
S3method(date_ceiling,Date)
S3method(date_ceiling,POSIXt)
S3method(date_count_between,Date)
S3method(date_count_between,POSIXt)
S3method(date_end,Date)
S3method(date_end,POSIXt)
S3method(date_floor,Date)
S3method(date_floor,POSIXt)
S3method(date_format,Date)
S3method(date_format,POSIXt)
S3method(date_group,Date)
S3method(date_group,POSIXt)
S3method(date_leap_year,Date)
S3method(date_leap_year,POSIXt)
S3method(date_round,Date)
S3method(date_round,POSIXt)
S3method(date_seq,Date)
S3method(date_seq,POSIXt)
S3method(date_set_zone,Date)
S3method(date_set_zone,POSIXt)
S3method(date_shift,Date)
S3method(date_shift,POSIXt)
S3method(date_spanning_seq,Date)
S3method(date_spanning_seq,POSIXt)
S3method(date_start,Date)
S3method(date_start,POSIXt)
S3method(date_zone,Date)
S3method(date_zone,POSIXt)
S3method(format,clock_duration)
S3method(format,clock_iso_year_week_day)
S3method(format,clock_time_point)
S3method(format,clock_weekday)
S3method(format,clock_year_day)
S3method(format,clock_year_month_day)
S3method(format,clock_year_month_weekday)
S3method(format,clock_year_quarter_day)
S3method(format,clock_year_week_day)
S3method(format,clock_zoned_time)
S3method(get_day,Date)
S3method(get_day,POSIXt)
S3method(get_day,clock_iso_year_week_day)
S3method(get_day,clock_year_day)
S3method(get_day,clock_year_month_day)
S3method(get_day,clock_year_month_weekday)
S3method(get_day,clock_year_quarter_day)
S3method(get_day,clock_year_week_day)
S3method(get_day,default)
S3method(get_hour,POSIXt)
S3method(get_hour,clock_iso_year_week_day)
S3method(get_hour,clock_year_day)
S3method(get_hour,clock_year_month_day)
S3method(get_hour,clock_year_month_weekday)
S3method(get_hour,clock_year_quarter_day)
S3method(get_hour,clock_year_week_day)
S3method(get_hour,default)
S3method(get_index,clock_year_month_weekday)
S3method(get_index,default)
S3method(get_microsecond,clock_iso_year_week_day)
S3method(get_microsecond,clock_year_day)
S3method(get_microsecond,clock_year_month_day)
S3method(get_microsecond,clock_year_month_weekday)
S3method(get_microsecond,clock_year_quarter_day)
S3method(get_microsecond,clock_year_week_day)
S3method(get_microsecond,default)
S3method(get_millisecond,clock_iso_year_week_day)
S3method(get_millisecond,clock_year_day)
S3method(get_millisecond,clock_year_month_day)
S3method(get_millisecond,clock_year_month_weekday)
S3method(get_millisecond,clock_year_quarter_day)
S3method(get_millisecond,clock_year_week_day)
S3method(get_millisecond,default)
S3method(get_minute,POSIXt)
S3method(get_minute,clock_iso_year_week_day)
S3method(get_minute,clock_year_day)
S3method(get_minute,clock_year_month_day)
S3method(get_minute,clock_year_month_weekday)
S3method(get_minute,clock_year_quarter_day)
S3method(get_minute,clock_year_week_day)
S3method(get_minute,default)
S3method(get_month,Date)
S3method(get_month,POSIXt)
S3method(get_month,clock_year_month_day)
S3method(get_month,clock_year_month_weekday)
S3method(get_month,default)
S3method(get_nanosecond,clock_iso_year_week_day)
S3method(get_nanosecond,clock_year_day)
S3method(get_nanosecond,clock_year_month_day)
S3method(get_nanosecond,clock_year_month_weekday)
S3method(get_nanosecond,clock_year_quarter_day)
S3method(get_nanosecond,clock_year_week_day)
S3method(get_nanosecond,default)
S3method(get_quarter,clock_year_quarter_day)
S3method(get_quarter,default)
S3method(get_second,POSIXt)
S3method(get_second,clock_iso_year_week_day)
S3method(get_second,clock_year_day)
S3method(get_second,clock_year_month_day)
S3method(get_second,clock_year_month_weekday)
S3method(get_second,clock_year_quarter_day)
S3method(get_second,clock_year_week_day)
S3method(get_second,default)
S3method(get_week,clock_iso_year_week_day)
S3method(get_week,clock_year_week_day)
S3method(get_week,default)
S3method(get_year,Date)
S3method(get_year,POSIXt)
S3method(get_year,clock_iso_year_week_day)
S3method(get_year,clock_year_day)
S3method(get_year,clock_year_month_day)
S3method(get_year,clock_year_month_weekday)
S3method(get_year,clock_year_quarter_day)
S3method(get_year,clock_year_week_day)
S3method(get_year,default)
S3method(invalid_any,clock_iso_year_week_day)
S3method(invalid_any,clock_year_day)
S3method(invalid_any,clock_year_month_day)
S3method(invalid_any,clock_year_month_weekday)
S3method(invalid_any,clock_year_quarter_day)
S3method(invalid_any,clock_year_week_day)
S3method(invalid_any,default)
S3method(invalid_count,clock_iso_year_week_day)
S3method(invalid_count,clock_year_day)
S3method(invalid_count,clock_year_month_day)
S3method(invalid_count,clock_year_month_weekday)
S3method(invalid_count,clock_year_quarter_day)
S3method(invalid_count,clock_year_week_day)
S3method(invalid_count,default)
S3method(invalid_detect,clock_iso_year_week_day)
S3method(invalid_detect,clock_year_day)
S3method(invalid_detect,clock_year_month_day)
S3method(invalid_detect,clock_year_month_weekday)
S3method(invalid_detect,clock_year_quarter_day)
S3method(invalid_detect,clock_year_week_day)
S3method(invalid_detect,default)
S3method(invalid_remove,clock_calendar)
S3method(invalid_remove,default)
S3method(invalid_resolve,clock_iso_year_week_day)
S3method(invalid_resolve,clock_year_day)
S3method(invalid_resolve,clock_year_month_day)
S3method(invalid_resolve,clock_year_month_weekday)
S3method(invalid_resolve,clock_year_quarter_day)
S3method(invalid_resolve,clock_year_week_day)
S3method(invalid_resolve,default)
S3method(max,clock_rcrd)
S3method(min,clock_rcrd)
S3method(names,clock_rcrd)
S3method(obj_print_data,clock_calendar)
S3method(obj_print_data,clock_duration)
S3method(obj_print_data,clock_time_point)
S3method(obj_print_data,clock_zoned_time)
S3method(obj_print_footer,clock_calendar)
S3method(obj_print_footer,clock_time_point)
S3method(obj_print_footer,clock_zoned_time)
S3method(print,clock_calendar)
S3method(print,clock_labels)
S3method(print,clock_locale)
S3method(print,clock_time_point)
S3method(print,clock_zoned_time)
S3method(range,clock_rcrd)
S3method(seq,clock_duration)
S3method(seq,clock_iso_year_week_day)
S3method(seq,clock_time_point)
S3method(seq,clock_year_day)
S3method(seq,clock_year_month_day)
S3method(seq,clock_year_month_weekday)
S3method(seq,clock_year_quarter_day)
S3method(seq,clock_year_week_day)
S3method(set_day,Date)
S3method(set_day,POSIXt)
S3method(set_day,clock_iso_year_week_day)
S3method(set_day,clock_year_day)
S3method(set_day,clock_year_month_day)
S3method(set_day,clock_year_month_weekday)
S3method(set_day,clock_year_quarter_day)
S3method(set_day,clock_year_week_day)
S3method(set_day,default)
S3method(set_hour,POSIXt)
S3method(set_hour,clock_iso_year_week_day)
S3method(set_hour,clock_year_day)
S3method(set_hour,clock_year_month_day)
S3method(set_hour,clock_year_month_weekday)
S3method(set_hour,clock_year_quarter_day)
S3method(set_hour,clock_year_week_day)
S3method(set_hour,default)
S3method(set_index,clock_year_month_weekday)
S3method(set_index,default)
S3method(set_microsecond,clock_iso_year_week_day)
S3method(set_microsecond,clock_year_day)
S3method(set_microsecond,clock_year_month_day)
S3method(set_microsecond,clock_year_month_weekday)
S3method(set_microsecond,clock_year_quarter_day)
S3method(set_microsecond,clock_year_week_day)
S3method(set_microsecond,default)
S3method(set_millisecond,clock_iso_year_week_day)
S3method(set_millisecond,clock_year_day)
S3method(set_millisecond,clock_year_month_day)
S3method(set_millisecond,clock_year_month_weekday)
S3method(set_millisecond,clock_year_quarter_day)
S3method(set_millisecond,clock_year_week_day)
S3method(set_millisecond,default)
S3method(set_minute,POSIXt)
S3method(set_minute,clock_iso_year_week_day)
S3method(set_minute,clock_year_day)
S3method(set_minute,clock_year_month_day)
S3method(set_minute,clock_year_month_weekday)
S3method(set_minute,clock_year_quarter_day)
S3method(set_minute,clock_year_week_day)
S3method(set_minute,default)
S3method(set_month,Date)
S3method(set_month,POSIXt)
S3method(set_month,clock_year_month_day)
S3method(set_month,clock_year_month_weekday)
S3method(set_month,default)
S3method(set_nanosecond,clock_iso_year_week_day)
S3method(set_nanosecond,clock_year_day)
S3method(set_nanosecond,clock_year_month_day)
S3method(set_nanosecond,clock_year_month_weekday)
S3method(set_nanosecond,clock_year_quarter_day)
S3method(set_nanosecond,clock_year_week_day)
S3method(set_nanosecond,default)
S3method(set_quarter,clock_year_quarter_day)
S3method(set_quarter,default)
S3method(set_second,POSIXt)
S3method(set_second,clock_iso_year_week_day)
S3method(set_second,clock_year_day)
S3method(set_second,clock_year_month_day)
S3method(set_second,clock_year_month_weekday)
S3method(set_second,clock_year_quarter_day)
S3method(set_second,clock_year_week_day)
S3method(set_second,default)
S3method(set_week,clock_iso_year_week_day)
S3method(set_week,clock_year_week_day)
S3method(set_week,default)
S3method(set_year,Date)
S3method(set_year,POSIXt)
S3method(set_year,clock_iso_year_week_day)
S3method(set_year,clock_year_day)
S3method(set_year,clock_year_month_day)
S3method(set_year,clock_year_month_weekday)
S3method(set_year,clock_year_quarter_day)
S3method(set_year,clock_year_week_day)
S3method(set_year,default)
S3method(vec_arith,clock_duration)
S3method(vec_arith,clock_iso_year_week_day)
S3method(vec_arith,clock_naive_time)
S3method(vec_arith,clock_sys_time)
S3method(vec_arith,clock_weekday)
S3method(vec_arith,clock_year_day)
S3method(vec_arith,clock_year_month_day)
S3method(vec_arith,clock_year_month_weekday)
S3method(vec_arith,clock_year_quarter_day)
S3method(vec_arith,clock_year_week_day)
S3method(vec_arith.Date,clock_duration)
S3method(vec_arith.POSIXct,clock_duration)
S3method(vec_arith.POSIXlt,clock_duration)
S3method(vec_arith.clock_duration,Date)
S3method(vec_arith.clock_duration,MISSING)
S3method(vec_arith.clock_duration,POSIXct)
S3method(vec_arith.clock_duration,POSIXlt)
S3method(vec_arith.clock_duration,clock_duration)
S3method(vec_arith.clock_duration,clock_iso_year_week_day)
S3method(vec_arith.clock_duration,clock_naive_time)
S3method(vec_arith.clock_duration,clock_sys_time)
S3method(vec_arith.clock_duration,clock_weekday)
S3method(vec_arith.clock_duration,clock_year_day)
S3method(vec_arith.clock_duration,clock_year_month_day)
S3method(vec_arith.clock_duration,clock_year_month_weekday)
S3method(vec_arith.clock_duration,clock_year_quarter_day)
S3method(vec_arith.clock_duration,clock_year_week_day)
S3method(vec_arith.clock_duration,numeric)
S3method(vec_arith.clock_iso_year_week_day,MISSING)
S3method(vec_arith.clock_iso_year_week_day,clock_duration)
S3method(vec_arith.clock_iso_year_week_day,clock_iso_year_week_day)
S3method(vec_arith.clock_iso_year_week_day,numeric)
S3method(vec_arith.clock_naive_time,MISSING)
S3method(vec_arith.clock_naive_time,clock_duration)
S3method(vec_arith.clock_naive_time,clock_naive_time)
S3method(vec_arith.clock_naive_time,numeric)
S3method(vec_arith.clock_sys_time,MISSING)
S3method(vec_arith.clock_sys_time,clock_duration)
S3method(vec_arith.clock_sys_time,clock_sys_time)
S3method(vec_arith.clock_sys_time,numeric)
S3method(vec_arith.clock_weekday,MISSING)
S3method(vec_arith.clock_weekday,clock_duration)
S3method(vec_arith.clock_weekday,clock_weekday)
S3method(vec_arith.clock_weekday,numeric)
S3method(vec_arith.clock_year_day,MISSING)
S3method(vec_arith.clock_year_day,clock_duration)
S3method(vec_arith.clock_year_day,clock_year_day)
S3method(vec_arith.clock_year_day,numeric)
S3method(vec_arith.clock_year_month_day,MISSING)
S3method(vec_arith.clock_year_month_day,clock_duration)
S3method(vec_arith.clock_year_month_day,clock_year_month_day)
S3method(vec_arith.clock_year_month_day,numeric)
S3method(vec_arith.clock_year_month_weekday,MISSING)
S3method(vec_arith.clock_year_month_weekday,clock_duration)
S3method(vec_arith.clock_year_month_weekday,clock_year_month_weekday)
S3method(vec_arith.clock_year_month_weekday,numeric)
S3method(vec_arith.clock_year_quarter_day,MISSING)
S3method(vec_arith.clock_year_quarter_day,clock_duration)
S3method(vec_arith.clock_year_quarter_day,clock_year_quarter_day)
S3method(vec_arith.clock_year_quarter_day,numeric)
S3method(vec_arith.clock_year_week_day,MISSING)
S3method(vec_arith.clock_year_week_day,clock_duration)
S3method(vec_arith.clock_year_week_day,clock_year_week_day)
S3method(vec_arith.clock_year_week_day,numeric)
S3method(vec_arith.numeric,clock_duration)
S3method(vec_arith.numeric,clock_iso_year_week_day)
S3method(vec_arith.numeric,clock_naive_time)
S3method(vec_arith.numeric,clock_sys_time)
S3method(vec_arith.numeric,clock_weekday)
S3method(vec_arith.numeric,clock_year_day)
S3method(vec_arith.numeric,clock_year_month_day)
S3method(vec_arith.numeric,clock_year_month_weekday)
S3method(vec_arith.numeric,clock_year_quarter_day)
S3method(vec_arith.numeric,clock_year_week_day)
S3method(vec_cast,clock_duration.clock_duration)
S3method(vec_cast,clock_iso_year_week_day.clock_iso_year_week_day)
S3method(vec_cast,clock_naive_time.clock_naive_time)
S3method(vec_cast,clock_sys_time.clock_sys_time)
S3method(vec_cast,clock_weekday.clock_weekday)
S3method(vec_cast,clock_year_day.clock_year_day)
S3method(vec_cast,clock_year_month_day.clock_year_month_day)
S3method(vec_cast,clock_year_month_weekday.clock_year_month_weekday)
S3method(vec_cast,clock_year_quarter_day.clock_year_quarter_day)
S3method(vec_cast,clock_year_week_day.clock_year_week_day)
S3method(vec_cast,clock_zoned_time.clock_zoned_time)
S3method(vec_math,clock_duration)
S3method(vec_math,clock_rcrd)
S3method(vec_math,clock_weekday)
S3method(vec_proxy,clock_duration)
S3method(vec_proxy,clock_iso_year_week_day)
S3method(vec_proxy,clock_time_point)
S3method(vec_proxy,clock_year_day)
S3method(vec_proxy,clock_year_month_day)
S3method(vec_proxy,clock_year_month_weekday)
S3method(vec_proxy,clock_year_quarter_day)
S3method(vec_proxy,clock_year_week_day)
S3method(vec_proxy,clock_zoned_time)
S3method(vec_proxy_compare,clock_weekday)
S3method(vec_proxy_compare,clock_year_month_weekday)
S3method(vec_ptype,clock_iso_year_week_day)
S3method(vec_ptype,clock_naive_time)
S3method(vec_ptype,clock_sys_time)
S3method(vec_ptype,clock_weekday)
S3method(vec_ptype,clock_year_day)
S3method(vec_ptype,clock_year_month_day)
S3method(vec_ptype,clock_year_month_weekday)
S3method(vec_ptype,clock_year_quarter_day)
S3method(vec_ptype,clock_year_week_day)
S3method(vec_ptype,clock_zoned_time)
S3method(vec_ptype2,clock_duration.clock_duration)
S3method(vec_ptype2,clock_iso_year_week_day.clock_iso_year_week_day)
S3method(vec_ptype2,clock_naive_time.clock_naive_time)
S3method(vec_ptype2,clock_sys_time.clock_sys_time)
S3method(vec_ptype2,clock_weekday.clock_weekday)
S3method(vec_ptype2,clock_year_day.clock_year_day)
S3method(vec_ptype2,clock_year_month_day.clock_year_month_day)
S3method(vec_ptype2,clock_year_month_weekday.clock_year_month_weekday)
S3method(vec_ptype2,clock_year_quarter_day.clock_year_quarter_day)
S3method(vec_ptype2,clock_year_week_day.clock_year_week_day)
S3method(vec_ptype2,clock_zoned_time.clock_zoned_time)
S3method(vec_ptype_abbr,clock_duration)
S3method(vec_ptype_abbr,clock_iso_year_week_day)
S3method(vec_ptype_abbr,clock_naive_time)
S3method(vec_ptype_abbr,clock_sys_time)
S3method(vec_ptype_abbr,clock_weekday)
S3method(vec_ptype_abbr,clock_year_day)
S3method(vec_ptype_abbr,clock_year_month_day)
S3method(vec_ptype_abbr,clock_year_month_weekday)
S3method(vec_ptype_abbr,clock_year_quarter_day)
S3method(vec_ptype_abbr,clock_year_week_day)
S3method(vec_ptype_abbr,clock_zoned_time)
S3method(vec_ptype_full,clock_duration)
S3method(vec_ptype_full,clock_iso_year_week_day)
S3method(vec_ptype_full,clock_naive_time)
S3method(vec_ptype_full,clock_sys_time)
S3method(vec_ptype_full,clock_weekday)
S3method(vec_ptype_full,clock_year_day)
S3method(vec_ptype_full,clock_year_month_day)
S3method(vec_ptype_full,clock_year_month_weekday)
S3method(vec_ptype_full,clock_year_quarter_day)
S3method(vec_ptype_full,clock_year_week_day)
S3method(vec_ptype_full,clock_zoned_time)
S3method(vec_restore,clock_duration)
S3method(vec_restore,clock_iso_year_week_day)
S3method(vec_restore,clock_time_point)
S3method(vec_restore,clock_year_day)
S3method(vec_restore,clock_year_month_day)
S3method(vec_restore,clock_year_month_weekday)
S3method(vec_restore,clock_year_quarter_day)
S3method(vec_restore,clock_year_week_day)
S3method(vec_restore,clock_zoned_time)
export(add_days)
export(add_hours)
export(add_microseconds)
export(add_milliseconds)
export(add_minutes)
export(add_months)
export(add_nanoseconds)
export(add_quarters)
export(add_seconds)
export(add_weeks)
export(add_years)
export(as_date)
export(as_date_time)
export(as_duration)
export(as_iso_year_week_day)
export(as_naive_time)
export(as_sys_time)
export(as_weekday)
export(as_year_day)
export(as_year_month_day)
export(as_year_month_weekday)
export(as_year_quarter_day)
export(as_year_week_day)
export(as_zoned_time)
export(calendar_count_between)
export(calendar_end)
export(calendar_group)
export(calendar_leap_year)
export(calendar_month_factor)
export(calendar_narrow)
export(calendar_precision)
export(calendar_spanning_seq)
export(calendar_start)
export(calendar_widen)
export(clock_labels)
export(clock_labels_languages)
export(clock_labels_lookup)
export(clock_locale)
export(date_build)
export(date_ceiling)
export(date_count_between)
export(date_end)
export(date_floor)
export(date_format)
export(date_group)
export(date_leap_year)
export(date_month_factor)
export(date_now)
export(date_parse)
export(date_round)
export(date_seq)
export(date_set_zone)
export(date_shift)
export(date_spanning_seq)
export(date_start)
export(date_time_build)
export(date_time_info)
export(date_time_parse)
export(date_time_parse_RFC_3339)
export(date_time_parse_abbrev)
export(date_time_parse_complete)
export(date_time_set_zone)
export(date_time_zone)
export(date_today)
export(date_weekday_factor)
export(date_zone)
export(duration_cast)
export(duration_ceiling)
export(duration_days)
export(duration_floor)
export(duration_hours)
export(duration_microseconds)
export(duration_milliseconds)
export(duration_minutes)
export(duration_months)
export(duration_nanoseconds)
export(duration_precision)
export(duration_quarters)
export(duration_round)
export(duration_seconds)
export(duration_spanning_seq)
export(duration_weeks)
export(duration_years)
export(get_day)
export(get_hour)
export(get_index)
export(get_microsecond)
export(get_millisecond)
export(get_minute)
export(get_month)
export(get_nanosecond)
export(get_quarter)
export(get_second)
export(get_week)
export(get_year)
export(invalid_any)
export(invalid_count)
export(invalid_detect)
export(invalid_remove)
export(invalid_resolve)
export(is_duration)
export(is_iso_year_week_day)
export(is_naive_time)
export(is_sys_time)
export(is_weekday)
export(is_year_day)
export(is_year_month_day)
export(is_year_month_weekday)
export(is_year_quarter_day)
export(is_year_week_day)
export(is_zoned_time)
export(iso_year_week_day)
export(naive_time_info)
export(naive_time_parse)
export(set_day)
export(set_hour)
export(set_index)
export(set_microsecond)
export(set_millisecond)
export(set_minute)
export(set_month)
export(set_nanosecond)
export(set_quarter)
export(set_second)
export(set_week)
export(set_year)
export(sys_time_info)
export(sys_time_now)
export(sys_time_parse)
export(sys_time_parse_RFC_3339)
export(time_point_cast)
export(time_point_ceiling)
export(time_point_count_between)
export(time_point_floor)
export(time_point_precision)
export(time_point_round)
export(time_point_shift)
export(time_point_spanning_seq)
export(tzdb_names)
export(tzdb_version)
export(weekday)
export(weekday_code)
export(weekday_factor)
export(year_day)
export(year_month_day)
export(year_month_day_parse)
export(year_month_weekday)
export(year_quarter_day)
export(year_week_day)
export(zoned_time_info)
export(zoned_time_now)
export(zoned_time_parse_abbrev)
export(zoned_time_parse_complete)
export(zoned_time_precision)
export(zoned_time_set_zone)
export(zoned_time_zone)
import(rlang)
import(vctrs)
importFrom(tzdb,tzdb_names)
importFrom(tzdb,tzdb_version)
useDynLib(clock, .registration = TRUE)
