% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sys-time.R
\name{is_sys_time}
\alias{is_sys_time}
\title{Is \code{x} a sys-time?}
\usage{
is_sys_time(x)
}
\arguments{
\item{x}{\verb{[object]}

An object.}
}
\value{
\code{TRUE} if \code{x} inherits from \code{"clock_sys_time"}, otherwise \code{FALSE}.
}
\description{
This function determines if the input is a sys-time object.
}
\examples{
is_sys_time(1)
is_sys_time(as_sys_time(duration_days(1)))
}
